﻿Critline.template = {}
local template = Critline.template
local core = Critline

local MAXSPELLBUTTONS = 14


-- this is used for creating the scroll frames for the Reset and Announce frames
function template:CreateList(name)
	local frame = CreateFrame("Frame")

	frame.Update = self.Update
	local function update()
		frame:Update()
	end
	
	frame.scrollFrame = CreateFrame("ScrollFrame", name.."ScrollFrame", frame, "FauxScrollFrameTemplate")
	frame.scrollFrame:SetWidth(300)
	frame.scrollFrame:SetHeight(MAXSPELLBUTTONS * 22 + 4)
	frame.scrollFrame:SetPoint("TOP", 0, -24)
	frame.scrollFrame:SetScript("OnShow", function(self)
		wipe(frame.selectedSpells)
		FauxScrollFrame_SetOffset(self, 0)
		_G[self:GetName().."ScrollBar"]:SetValue(0)
		frame:Update()
	end)
	frame.scrollFrame:SetScript("OnVerticalScroll", function(self, offset)
		FauxScrollFrame_OnVerticalScroll(self, offset, 22, update)
	end)
	
	-- onClick for check buttons
	local function onClick(self)
		local _, pos = core:GetSpellFromDB(frame.tree.selectedValue, self.spell, self.isPeriodic)
		if self:GetChecked() then
			PlaySound("igMainMenuOptionCheckBoxOn")
			frame.selectedSpells[pos] = true
		else
			PlaySound("igMainMenuOptionCheckBoxOff")
			frame.selectedSpells[pos] = nil
		end
	end
	
	-- create list of check buttons
	frame.buttons = {}
	for i = 1, MAXSPELLBUTTONS do
		local button = CreateFrame("CheckButton", nil, frame, "OptionsBaseCheckButtonTemplate")
		if i == 1 then
			button:SetPoint("TOPLEFT", frame.scrollFrame)
		else
			button:SetPoint("TOP", frame.buttons[i - 1], "BOTTOM", 0, 4)
		end
		button:SetPushedTextOffset(0, 0)
		button:SetScript("OnClick", onClick)
		
		-- set default font string for the check button (will contain the spell name)
		local fontString = button:CreateFontString(nil, nil, "GameFontHighlight")
		fontString:SetPoint("LEFT", button, "RIGHT", 0, 1)
		fontString:SetJustifyH("LEFT")
		button:SetFontString(fontString)
		
		-- font string for record amounts
		button.record = button:CreateFontString(nil, nil, "GameFontHighlight")
		button.record:SetPoint("CENTER", fontString)
		button.record:SetPoint("RIGHT", frame.scrollFrame)
		button.record:SetJustifyH("RIGHT")
		
		frame.buttons[i] = button
	end
	
	frame.tree = CreateFrame("Frame", name.."Tree", frame, "UIDropDownMenuTemplate")
	frame.tree:SetPoint("TOPLEFT", frame.scrollFrame, "BOTTOMLEFT", -16, -4)
	
	local function onClick(self)
		UIDropDownMenu_SetSelectedValue(frame.tree, self.value)
		wipe(frame.selectedSpells)
		FauxScrollFrame_SetOffset(frame.scrollFrame, 0)
		_G[frame.scrollFrame:GetName().."ScrollBar"]:SetValue(0)
		frame:Update()
	end
	
	local info = {}
	
	UIDropDownMenu_Initialize(frame.tree, function(self)
		wipe(info)
		info.text = DAMAGE
		info.value = "dmg"
		info.func = onClick
		UIDropDownMenu_AddButton(info)
		
		wipe(info)
		info.text = HEALS
		info.value = "heal"
		info.func = onClick
		UIDropDownMenu_AddButton(info)

		wipe(info)
		info.text = PET
		info.value = "pet"
		info.func = onClick
		UIDropDownMenu_AddButton(info)
	end)
	
	UIDropDownMenu_SetWidth(frame.tree, 120)
	UIDropDownMenu_SetSelectedValue(frame.tree, "dmg")
	
	-- reset/announce button
	frame.button = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	frame.button:SetWidth(100)
	frame.button:SetHeight(24)
	frame.button:SetPoint("TOPRIGHT", frame.scrollFrame, "BOTTOMRIGHT", 0, -6)
	
	return frame
end


local spells = {}

function template:Update()
	local selectedTree = UIDropDownMenu_GetSelectedValue(self.tree)
	
	wipe(spells)
	
	for _, v in ipairs(CritlineDB[selectedTree]) do
		if v.normal or v.crit then
			spells[#spells + 1] = v
		end
	end
	
	local size = #spells
	
	FauxScrollFrame_Update(self.scrollFrame, size, MAXSPELLBUTTONS, 22)
	
	local offset = FauxScrollFrame_GetOffset(self.scrollFrame)
	
	for line = 1, MAXSPELLBUTTONS do
		local button = self.buttons[line]
		local lineplusoffset = line + offset
		if lineplusoffset <= size then
			local data = spells[lineplusoffset]
			button.spell = data.spellName
			button.isPeriodic = data.isPeriodic
			local normal = data.normal and data.normal.amount
			local crit = data.crit and data.crit.amount
			button:SetText(core:GetFullSpellName(selectedTree, data.spellName, data.isPeriodic))
			button.record:SetText(format("%d/%d", (normal or 0), (crit or 0)))
			button:SetChecked(self.selectedSpells[lineplusoffset])
			button:Show()
		else
			button:Hide()
		end
	end
end